import { WebComponent } from '../../../../webComponent/WebComponent';
import { PresentationMode, WebComponentType } from '../../../../types/internal/webComponent.types';
import { getBottomSheetScript, injectHtmlFullScreenHeight, renderingBottomSheetRequest } from '../../UNBottomSheetComponent.utils';
import React, { useEffect, useRef } from 'react';
import { PageMessage } from '../../../../messages/webMessages/pageMessage';
import { UnitComponentsMessage } from '../../../../messages/webMessages/unitComponentsMessages';
import { BottomSheetNativePlaceType, BottomSheetRenderingType, ScrollState } from '../../../../types/internal/bottomSheet.types';
import { CardMessage } from '../../../../messages/webMessages/cardMessage';
import { setEvent } from '../../../../slices/SharedEventsSlice';
import { AccountMessage } from '../../../../messages/webMessages/accountMessage';
import { ActivityMessage } from '../../../../messages/webMessages/activityMessage';
import { useDispatch, useSelector } from 'react-redux';
import { setComponentHeight, setIsComponentLoading, setScrollState, setShouldEnableBottomSheetScroll } from '../../../../slices/BottomSheetSlice';
import { getAccountActionType } from '../../../UNAccountComponent/UNAccountComponent.utils';
import { UNAccountAction } from '../../../UNAccountComponent/UNAccountComponent';
import { fullScreenHeight, overFullScreenHeight, statusBarHeight } from '../../UNBottomSheetComponent.constants';
import { BottomSheetNativeMessage } from '../../../../messages/nativeMessages/bottomSheetMessage';
import { PlaidMessage } from '../../../../messages/webMessages/plaidMessages';
import { PayeeManagementMessage } from '../../../../messages/webMessages/payeeManagementMessage';
import UNKeyboardAvoidingView from '../../../../helpers/UNKeyboardAvoidingView/UNKeyboardAvoidingView';
import { MultipleCardsMessage } from '../../../../messages/webMessages/multipleCardsMessage';
const UNBottomSheetSlotComponent = props => {
  const dispatch = useDispatch();
  const webRef = useRef(null);
  const scrollState = useSelector(state => state.bottomSheet.scrollState);
  const nativePlace = props.requestRenderingEvent?.data?.nativePlace;
  useEffect(() => {
    const scrollStateForSlot = () => {
      if (!props.requestRenderingEvent) return;
      switch (props.componentName) {
        case WebComponentType.account:
          {
            const accountActionType = getAccountActionType(props.requestRenderingEvent);
            dispatch(setScrollState(accountActionType === UNAccountAction.List ? ScrollState.onTop : ScrollState.unScrollable));
            dispatch(setShouldEnableBottomSheetScroll(true));
            break;
          }
        case WebComponentType.card:
          {
            dispatch(setScrollState(ScrollState.onTop));
            dispatch(setShouldEnableBottomSheetScroll(true));
            break;
          }
        default:
          {
            // default web component case
            const nativePlace = props.requestRenderingEvent.data.nativePlace;
            switch (nativePlace) {
              case BottomSheetNativePlaceType.overFullScreen:
                dispatch(setScrollState(ScrollState.onTop));
                dispatch(setShouldEnableBottomSheetScroll(true));
                break;
              case BottomSheetNativePlaceType.modal:
                dispatch(setScrollState(ScrollState.onlyWebView));
                dispatch(setShouldEnableBottomSheetScroll(false));
                break;
              default:
                dispatch(setScrollState(ScrollState.onTop));
                dispatch(setShouldEnableBottomSheetScroll(true));
                break;
            }
          }
      }
    };
    scrollStateForSlot();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [props.componentName, props.requestRenderingEvent.data.nativePlace]);
  const _onWebMessage = message => {
    switch (message.type) {
      case PageMessage.PAGE_LOADED:
        if (webRef.current && props.requestRenderingEvent) {
          renderingBottomSheetRequest(webRef.current, props.requestRenderingEvent);
        }
        break;
      case PageMessage.PAGE_HEIGHT:
        {
          const height = message.details.height;
          switch (nativePlace) {
            case BottomSheetNativePlaceType.overFullScreen:
              injectHtmlFullScreenHeight(webRef.current, overFullScreenHeight);
              break;
            case BottomSheetNativePlaceType.modal:
              injectHtmlFullScreenHeight(webRef.current, fullScreenHeight - statusBarHeight);
              break;
            case BottomSheetNativePlaceType.menu:
            case BottomSheetNativePlaceType.contentHeight:
              if (height < overFullScreenHeight) {
                dispatch(setComponentHeight(height));
              } else {
                dispatch(setComponentHeight(overFullScreenHeight));
              }
              break;
          }
          dispatch(setIsComponentLoading(false));
          break;
        }
      case UnitComponentsMessage.UNIT_REQUEST_RENDERING:
        {
          const requestRendering = message.details;
          const requestRenderingSlotData = {
            type: BottomSheetRenderingType.Slot,
            data: {
              requestRenderingEvent: requestRendering,
              componentResourceId: props.componentResourceId
            }
          };
          dispatch(setEvent({
            key: BottomSheetNativeMessage.REQUEST_RENDERING,
            data: requestRenderingSlotData
          }));
          break;
        }
      case UnitComponentsMessage.UNIT_REQUEST_CLOSE_FLOW:
        dispatch(setEvent({
          key: UnitComponentsMessage.UNIT_REQUEST_CLOSE_FLOW,
          data: {}
        }));
        break;
      case CardMessage.CARD_ACTIVATED:
        dispatch(setEvent({
          key: CardMessage.CARD_ACTIVATED,
          data: message.details.card
        }));
        break;
      case PlaidMessage.UNIT_OPEN_PLAID:
        dispatch(setEvent({
          key: PlaidMessage.UNIT_OPEN_PLAID,
          data: message.details
        }));
        break;
      case CardMessage.CARD_STATUS_CHANGED:
        dispatch(setEvent({
          key: CardMessage.CARD_STATUS_CHANGED,
          data: message.details.card
        }));
        break;
      case AccountMessage.UNIT_ACCOUNT_CHANGED:
        dispatch(setEvent({
          key: AccountMessage.UNIT_ACCOUNT_CHANGED,
          data: message.details.account
        }));
        break;
      case ActivityMessage.UNIT_ACTIVITY_FILTERS_CHANGED:
        dispatch(setEvent({
          key: ActivityMessage.UNIT_ACTIVITY_FILTERS_CHANGED,
          data: message.details
        }));
        break;
      case MultipleCardsMessage.UNIT_MULTIPLE_CARDS_FILTERS_CHANGED:
        dispatch(setEvent({
          key: MultipleCardsMessage.UNIT_MULTIPLE_CARDS_FILTERS_CHANGED,
          data: message.details
        }));
        break;
      case PayeeManagementMessage.UNIT_COUNTERPARTY_DELETED:
        {
          dispatch(setEvent({
            key: PayeeManagementMessage.UNIT_COUNTERPARTY_DELETED,
            data: message.details.counterparty
          }));
          break;
        }
    }
  };

  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  const _onScrollWebView = event => {
    if (scrollState === ScrollState.unScrollable) return;
    const currY = event.nativeEvent.contentOffset.y;
    if (ScrollState.onTop && currY < 5) {
      dispatch(setScrollState(ScrollState.onlyBottomSheet));
      dispatch(setShouldEnableBottomSheetScroll(true));
    }
    if (scrollState !== ScrollState.onlyWebView && scrollState === ScrollState.onTop && currY > 5) {
      dispatch(setScrollState(ScrollState.onlyWebView));
      dispatch(setShouldEnableBottomSheetScroll(false));
    }
  };
  return /*#__PURE__*/React.createElement(UNKeyboardAvoidingView, {
    keyboardVerticalOffset: fullScreenHeight - overFullScreenHeight
  }, /*#__PURE__*/React.createElement(WebComponent, {
    ref: webRef,
    type: WebComponentType.slot,
    presentationMode: PresentationMode.CoverInjectedHeight,
    script: getBottomSheetScript(),
    onMessage: _onWebMessage,
    isScrollable: scrollState === ScrollState.onlyWebView || scrollState === ScrollState.onTop,
    handleScroll: _onScrollWebView
  }));
};
export default UNBottomSheetSlotComponent;
//# sourceMappingURL=UNBottomSheetSlotComponent.js.map