import React, { useEffect, useState } from 'react';
import {
  BottomSheetNativeComponentType, BottomSheetNativePlaceType,
  NativeComponentData,
  ScrollState,
} from '../../../../types/internal/bottomSheet.types';
import { useDispatch } from 'react-redux';
import { setComponentHeight, setScrollState, setShouldEnableBottomSheetScroll } from '../../../../slices/BottomSheetSlice';
import { UNAddToWalletComponent } from '../../../../nativeComponents/UNAddToWallet/UNAddToWalletComponent';
import { fullScreenHeight, overFullScreenHeight, statusBarHeight } from '../../UNBottomSheetComponent.constants';
import UNACHCreditComponent from '../../../UNACHCreditComponent/UNACHCreditComponent';
import UNCheckDepositComponent from '../../../UNCheckDepositComponent/UNCheckDepositComponent';
import UNACHDebitComponent from '../../../UNACHDebitComponent/UNACHDebitComponent';
import UNBookPaymentComponent from '../../../UNBookPaymentComponent/UNBookPaymentComponent';
import UNWirePaymentComponent from '../../../UNWirePaymentComponent/UNWirePaymentComponent';
import UNCreateCardComponent from '../../../UNCreateCardComponent/UNCreateCardComponent';
import { PageMessage } from '../../../../messages/webMessages/pageMessage';
import { useListenerToEvent } from '../../../../hooks/useListenerToEvent';
import UNKeyboardAvoidingView from '../../../../helpers/UNKeyboardAvoidingView/UNKeyboardAvoidingView';

export interface UNBottomSheetNativeComponentProps {
  nativeComponentData: NativeComponentData;
}

const UNBottomSheetNativeComponent = (props: UNBottomSheetNativeComponentProps) => {
  const dispatch = useDispatch();
  const [height, setHeight] = useState<number>(0);
  const [paddingTop, setPaddingTop] = useState<number>(0);
  const [paddingBottom, setPaddingBottom] = useState<number>(0);
  const [padding, setPadding] = useState<number>(0);

  const updateComponetHeight = (height: number) => {
    // content height are supported for internal components only.
    setHeight(height);
    dispatch(setComponentHeight(height));
  };

  useListenerToEvent({ busEventKey: PageMessage.PAGE_HEIGHT, action: updateComponetHeight });

  useEffect(() => {
    const setPaddingForNativeComponent = () => {
      const type = props.nativeComponentData?.component.type;
      if (type != BottomSheetNativeComponentType.CheckDepositComponent) {
        setPadding(20);
        setPaddingTop(30);
      }
      setPaddingBottom(20);
    };
    const setScrollStateForNativeComponent = () => {
      const scrollState = props.nativeComponentData.scrollState ?? ScrollState.unScrollable;
      dispatch(setScrollState(scrollState));
      dispatch(setShouldEnableBottomSheetScroll(scrollState !== ScrollState.onlyWebView));
    };

    setScrollStateForNativeComponent();
    setPaddingForNativeComponent();
  }, [dispatch, props.nativeComponentData]);

  useEffect(() => {
    const setBottomSheetNativeComponentViewHeight = () => {
      let adjustedHeight: number | undefined;
      switch (props.nativeComponentData.nativePlace) {
        case BottomSheetNativePlaceType.overFullScreen:
          adjustedHeight = overFullScreenHeight;
          break;
        case BottomSheetNativePlaceType.modal: 
          adjustedHeight = fullScreenHeight - statusBarHeight;
          break;
        default:
          return;
      }
      if (adjustedHeight !== undefined) {
        setHeight(adjustedHeight);
        dispatch(setComponentHeight(adjustedHeight));
      }
    };

    setBottomSheetNativeComponentViewHeight();
  }, [props.nativeComponentData.nativePlace]);

  const renderNativeComponent = () => {
    switch (props.nativeComponentData?.component.type) {
      case BottomSheetNativeComponentType.AddToWalletComponent:
        return (
          <UNAddToWalletComponent
            {...props.nativeComponentData.component.props}
          />
        );
      case BottomSheetNativeComponentType.ACHCreditComponent:
        return (
          <UNACHCreditComponent
            {...props.nativeComponentData.component.props}
          />
        );
      case BottomSheetNativeComponentType.ACHDebitComponent:
        return (
          <UNACHDebitComponent
            {...props.nativeComponentData.component.props}
          />
        );
      case BottomSheetNativeComponentType.CheckDepositComponent:
        return (
          <UNCheckDepositComponent
            {...props.nativeComponentData.component.props}
          />
        );
      case BottomSheetNativeComponentType.BookPaymentComponent:
        return (
          <UNBookPaymentComponent
            {...props.nativeComponentData.component.props}
          />
        );
      case BottomSheetNativeComponentType.WirePaymentComponent:
        return (
          <UNWirePaymentComponent
            {...props.nativeComponentData.component.props}
          />
        );
      case BottomSheetNativeComponentType.CreateCardComponent:
        return (
          <UNCreateCardComponent
            {...props.nativeComponentData.component.props}
          />
        );
    }
  };

  switch (props.nativeComponentData.nativePlace) {
    case BottomSheetNativePlaceType.modal:
      return (
        <UNKeyboardAvoidingView style={{ height, padding, paddingTop, paddingBottom }}>
          {renderNativeComponent()}
        </UNKeyboardAvoidingView>
      );
    case BottomSheetNativePlaceType.overFullScreen:
      return (
        <UNKeyboardAvoidingView style={{ height }}>
          {renderNativeComponent()}
        </UNKeyboardAvoidingView>
      );
    default: 
      return (
        <UNKeyboardAvoidingView>
          {renderNativeComponent()}
        </UNKeyboardAvoidingView>
      );
  }
};

export default UNBottomSheetNativeComponent;
